
# q-expansion of newform 1274.2.g.b, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1274
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[885, [1, 0]], [197, [0, -1]]]
aps_data = [[1, -1], [-2, 2], [-1, 0], [0, 0], [-2, 2], [3, 1], [0, 1], [0, -4], [-2, 2], [5, -5], [-6, 0], [-7, 7], [-7, 7], [0, 2], [0, 0], [-9, 0], [0, -6], [0, 5], [-10, 10], [0, -16], [3, 0], [-10, 0], [-14, 0], [-6, 6], [0, 2], [-15, 15], [20, 0], [0, 0], [2, 0], [0, -19], [-4, 4], [0, 0], [0, 5], [0, -14], [0, 21], [8, 0], [11, 0], [0, 6], [-6, 6], [0, -10], [0, 0], [23, 0], [0, 6], [25, -25], [-18, 18], [0, -20], [14, -14], [-12, 12], [0, 10], [2, 0], [14, 0], [18, 0], [0, -19], [0, 20], [-15, 15], [14, -14], [0, 18], [-30, 30], [0, 17], [-29, 0], [-22, 22], [0, -7], [6, 0], [-30, 0], [-6, 0], [-25, 0], [0, 18], [27, 0], [0, 14], [2, -2], [25, -25], [-36, 0], [6, -6], [0, -19], [-34, 34], [0, -18], [23, 0], [0, 14], [-23, 23], [0, -31], [-10, 10], [19, 0], [-20, 20], [0, 9], [10, -10], [24, 0], [0, -30], [9, -9], [0, -11], [18, 0], [2, 0], [-6, 6], [0, -32], [-16, 16], [-14, 0], [0, 14], [-15, 15], [3, 0], [36, -36], [7, 0], [38, 0], [-7, 7], [0, 14], [-14, 14], [-40, 0], [39, 0], [12, -12], [19, 0], [-12, 0], [-31, 31], [0, -4], [-11, 11], [0, 5], [-26, 0], [0, 20], [0, 1], [0, 8], [-24, 24], [6, -6], [0, 48], [-23, 23], [-27, 27], [-22, 0], [0, 8], [40, -40], [-30, 0], [0, -39], [0, -22], [48, 0], [51, 0], [-16, 16], [-16, 16], [-10, 10], [-34, 34], [0, -14], [-14, 14], [0, -10], [0, -10], [0, -42], [45, -45], [-26, 0], [-22, 22], [0, 20], [34, 0], [-27, 27], [0, 12], [-25, 0], [3, 0], [50, 0], [-18, 0], [0, -23], [25, -25], [-34, 0], [8, -8], [42, -42], [12, 0], [0, -32], [0, 21], [-9, 0], [-14, 0], [-46, 46], [0, -38], [-42, 0], [0, 0], [21, -21], [-8, 0], [-30, 30], [0, 25], [-2, 0], [-41, 0], [38, 0], [0, -27], [0, -48], [0, 9], [18, 0], [0, -15], [0, 0], [27, 0], [-16, 16], [-58, 58], [-22, 0], [20, 0], [59, 0], [-2, 0], [36, -36], [0, 45], [-30, 0], [56, 0], [1, -1], [0, -40], [0, -11], [0, 34], [0, 0], [18, -18], [0, -62], [6, -6], [47, 0], [0, 57], [55, 0], [36, 0], [0, -27], [0, 44], [21, -21], [-1, 0], [32, -32], [65, -65], [-58, 0], [0, 6], [33, -33], [0, 44], [-19, 19], [26, 0], [32, -32], [0, 16], [0, 8], [-9, 0], [-2, 0], [0, 69], [36, -36], [-2, 0], [-7, 7], [-8, 0], [-45, 0], [0, -8], [12, -12], [-13, 0], [-51, 51], [0, 32], [0, 8], [-56, 0], [-31, 31], [16, -16], [-2, 2], [-41, 0], [-58, 0], [-24, 0], [0, -34], [41, -41], [0, -20], [14, -14], [-2, 2], [-14, 14], [0, -16], [-47, 47], [0, 53], [6, 0], [0, -74], [12, -12], [0, 10], [16, -16], [22, -22], [66, -66], [32, 0], [17, -17], [-70, 0], [0, -64], [0, 42], [-8, 8], [-38, 0], [0, 57], [16, 0], [-36, 36], [-61, 0], [-23, 23], [0, -31], [0, -26], [0, 6], [27, 0], [0, 24], [0, -23], [-29, 0], [-16, 0], [-59, 59], [0, 36], [0, -70], [34, -34], [0, -18], [-25, 0], [0, -39], [0, 76], [40, -40], [-4, 4], [42, 0], [-83, 83], [20, 0], [-80, 0], [-61, 0], [27, 0], [0, 4], [0, 33], [-46, 46], [0, -48], [57, -57], [0, -2], [0, -50], [-41, 0], [-48, 0], [-87, 87], [40, -40], [-26, 26], [0, 13], [27, 0], [0, 0]]
